package com.ningdatech.kqapi.irs.manage;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ningdatech.irs.service.IRefreshTokenService;
import com.ningdatech.irs.utils.HttpUtil;
import com.ningdatech.kqapi.common.util.CryptUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

/**
 * @author CMM
 * @author CMM
 * @date 2024/8/19 11:27
 * /
 * /**
 * @since 2024/08/19 11:27
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class IrsManage {

    @Value("${irs.item-basic-info.appKey}")
    private String appKey;

    @Value("${irs.item-basic-info.appSecret}")
    private String appSecret;

    @Value("${irs.item-basic-info.listUrl}")
    private String itemBasicInfoListUrl;

    @Value("${irs.item-basic-info.detailUrl}")
    private String itemBasicInfoDetailUrl;

    @Resource
    private IRefreshTokenService refreshTokenService;

    @Value("${irs.interface-refresh.request-token-url}")
    private String govRequestTokenUrl;

    @Value("${irs.interface-refresh.refresh-token-url}")
    private String govRefreshTokenUrl;

    public Object itemBasicInfoList(String startDate, String endDate, Integer pageSize, Integer pageNum) {
        long timestamp = System.currentTimeMillis();
        String secret = refreshTokenService.refreshToken(appKey, appSecret, govRequestTokenUrl, govRefreshTokenUrl,
                HttpMethod.POST.name());
        String sign = null;
        try{
            sign = CryptUtils.MD5Encode(appKey + secret + timestamp);
        }catch (Exception e){
            log.error(e.getMessage());
        }
        HttpComponentsClientHttpRequestFactory factory = HttpUtil.generateHttpRequestFactory();
        RestTemplate restTemplate;
        if(Objects.nonNull(factory)){
            restTemplate = new RestTemplate(factory);
        }else{
            restTemplate = new RestTemplate();
        }

        MultiValueMap<String, Object> map = new LinkedMultiValueMap<>();
        map.add("pageSize",pageSize);
        map.add("pageNum",pageNum);
        map.add("startDate",startDate);
        map.add("endDate",endDate);
        map.add("paramStr","update_date");
        map.add("viewName","v_dsc_sx_ads_share_item_qlt_qlsx_common_i_d_v_kq_insensitive");
        map.add("appKey",appKey);
        map.add("sign",sign);
        map.add("requestTime",timestamp);
        // 创建 HttpHeaders 对象
        HttpHeaders headers = new HttpHeaders();
        // 创建 HttpEntity 对象，并传入 HttpHeaders
        HttpEntity<Object> httpEntity = new HttpEntity<>(map,headers);
        log.info("请求参数 :{}", JSON.toJSONString(map));
        JSONObject jsonObject = restTemplate.postForObject(itemBasicInfoListUrl, httpEntity, JSONObject.class);
        log.info("return list data :{}",jsonObject);

        JSONArray dataArray = Objects.requireNonNull(jsonObject).getJSONArray("data");
        List<Object> dataList = new ArrayList<>(dataArray);
        if(CollUtil.isNotEmpty(dataList)){
            return dataList;
        } else {
            return jsonObject;
        }
    }

    public Object itemBasicInfoDetail(String itemId) {
        long timestamp = System.currentTimeMillis();
        String secret = refreshTokenService.refreshToken(appKey, appSecret, govRequestTokenUrl, govRefreshTokenUrl,
                HttpMethod.POST.name());
        String sign = null;
        try{
            sign = CryptUtils.MD5Encode(appKey + secret + timestamp);
        }catch (Exception e){
            log.error(e.getMessage());
        }
        HttpComponentsClientHttpRequestFactory factory = HttpUtil.generateHttpRequestFactory();
        RestTemplate restTemplate;
        if(Objects.nonNull(factory)){
            restTemplate = new RestTemplate(factory);
        }else{
            restTemplate = new RestTemplate();
        }

        MultiValueMap<String, Object> map = new LinkedMultiValueMap<>();
        // 附加信息
        map.add("paramStr",itemId);
        map.add("viewName","v_dsc_sx_ads_share_item_qlt_qlsx_common_i_d_v_kq_insensitive");
        map.add("appKey",appKey);
        map.add("sign",sign);
        map.add("requestTime",timestamp);
        // 创建 HttpHeaders 对象
        HttpHeaders headers = new HttpHeaders();
        // 创建 HttpEntity 对象，并传入 HttpHeaders
        HttpEntity<Object> httpEntity = new HttpEntity<>(map,headers);
        log.info("请求参数 :{}", JSON.toJSONString(map));
        JSONObject jsonObject = restTemplate.postForObject(itemBasicInfoDetailUrl, httpEntity, JSONObject.class);
        log.info("return detail data :{}",jsonObject);

        JSONArray dataArray = Objects.requireNonNull(jsonObject).getJSONArray("data");
        List<Object> dataList = new ArrayList<>(dataArray);
        if(CollUtil.isNotEmpty(dataList)){
            return dataList;
        } else {
            return jsonObject;
        }
    }
}
